@@ -80,8 +80,9 @@ enum {
 	EWS_PRIORITY_HIGH
 };
 
-typedef void	(*EEwsRequestCreationCallback)	(ESoapMessage *msg,
-						 gpointer user_data);
+typedef gboolean(*EEwsRequestCreationCallback)	(ESoapMessage *msg,
+						 gpointer user_data,
+						 GError **error);
 typedef void	(*EwsProgressFn)		(gpointer object,
 						 gint percent);
 typedef void	(*EEwsResponseCallback)		(ESoapResponse *response,
@@ -450,11 +451,6 @@ CamelEwsSettings *
 		e_ews_connection_ref_settings	(EEwsConnection *cnc);
 SoupSession *	e_ews_connection_ref_soup_session
 						(EEwsConnection *cnc);
-gboolean	e_ews_connection_get_disconnected_flag
-						(EEwsConnection *cnc);
-void		e_ews_connection_set_disconnected_flag
-						(EEwsConnection *cnc,
-						 gboolean disconnected_flag);
 EEwsConnection *e_ews_connection_find		(const gchar *uri,
 						 const gchar *username);
 GSList *	e_ews_connection_list_existing	(void); /* EEwsConnection * */
